// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
  "cloudera/cmf/wizard/WizardHiddenStepBase",
  "knockout",
  "underscore"
], function (WizardHiddenStepBase, ko, _) {
  "use strict";

  return WizardHiddenStepBase.extend({
    /**
     * options: {
     *   id:       (required) "the id of the step",
     *   createUrl:(required) "the URL to create a cluster",
     *   deleteUrl:(required) "the URL to delete a cluster",
     *   hosts:    (required) "a function that returns list of hosts to create new cluster with",
     *   version:  (required) "a function that returns CDH version of the cluster"
     * }
     *
     * Output:
     * cluster = the newly created cluster
     */
    init: function(options) {
      var self = this;
      self.options = options;
      self.cluster = ko.observable({});
      self.created = false;
      self._super.apply(self, arguments);
    },

    execute: function(callback, isForward) {
      if (!this.created && isForward) {
        this.createCluster(callback);
      } else if (this.created && !isForward) {
        this.deleteCluster(callback);
      } else {
        callback();
      }
    },

    createCluster: function(callback) {
      var self = this;
      var params = {
        version: self.options.version(),
        hosts: self.options.hosts()
      };
      $.post(self.options.createUrl, params, function(response) {
        if (response.message === "OK") {
          self.cluster().id = response.data.id;
          self.cluster().name = response.data.name;
          self.created = true;
        } else {
          $.publish("showError", [response.message]);
        }
      }, "json").complete(function() {
        // We should call callback regardless.
        if (_.isFunction(callback)) {
          callback();
        }
      });
    },

    deleteCluster: function(callback) {
      var self = this;
      var params = {
        clusterId: self.cluster().id
      };
      $.post(self.options.deleteUrl, params, function(response) {
        if (response.message === "OK") {
          self.cluster({});
          self.created = false;
        } else {
          $.publish("showError", [response.message]);
        }
      }, "json").complete(function() {
        // We should call callback regardless.
        if (_.isFunction(callback)) {
          callback();
        }
      });
    }

  });
});
